@@ -8,7 +8,7 @@ class Puppet::Indirector::Exec < Puppet::Indirector::Terminus
     external_command = command
 
     # Make sure it's an array
-    raise Puppet::DevError, "Exec commands must be an array" unless external_command.is_a?(Array)
+    raise Puppet::DevError, _("Exec commands must be an array") unless external_command.is_a?(Array)
 
     # Make sure it's fully qualified.
     raise ArgumentError, _("You must set the exec parameter to a fully qualified command") unless Puppet::Util.absolute_path?(external_command[0])
