@@ -96,7 +96,7 @@ class Puppet::Interface
         if Puppet::Interface::FaceCollection[name, :current]
           raise Puppet::Error, "Could not find version #{version} of #{name}"
         else
-          raise Puppet::Error, "Could not find Puppet Face #{name.to_s}"
+          raise Puppet::Error, "Could not find Puppet Face #{name}"
         end
       end
 
@@ -149,7 +149,7 @@ class Puppet::Interface
   # @api private
   def initialize(name, version, &block)
     unless SemanticPuppet::Version.valid?(version)
-      raise ArgumentError, "Cannot create face #{name.inspect} with invalid version number '#{version}'!"
+      raise ArgumentError, _("Cannot create face %{name} with invalid version number '%{version}'!") % { name: name.inspect, version: version }
     end
 
     @name    = Puppet::Interface::FaceCollection.underscorize(name)
@@ -178,6 +178,7 @@ class Puppet::Interface
   def to_s
     "Puppet::Face[#{name.inspect}, #{version.inspect}]"
   end
+  alias_method :inspect, :to_s
 
   # @return [void]
   def deprecate
@@ -233,4 +234,5 @@ class Puppet::Interface
     define_method(name, proc)
     instance_method(name)
   end
+  private_class_method :__add_method
 end
