@@ -39,5 +39,5 @@ $matches = ["abc123","def456"].match(/([a-z]+)([1-9]+)/)
 - Since 4.0.0
 DOC
 ) do |args|
-  function_fail(["match() is only available when parser/evaluator future is in effect"])
+  Puppet::Parser::Functions::Error.is4x('match')
 end
