@@ -3,11 +3,11 @@ Puppet::Parser::Functions::newfunction(
   :type => :rvalue,
   :arity => -1,
   :doc => <<-DOC
-Call a [lambda](https://docs.puppetlabs.com/puppet/latest/reference/lang_lambdas.html)
+Call a [lambda](https://puppet.com/docs/puppet/latest/lang_lambdas.html)
 with the given arguments and return the result. Since a lambda's scope is
-[local](https://docs.puppetlabs.com/puppet/latest/reference/lang_lambdas.html#lambda-scope)
-to the lambda, you can use the `with` function to create private blocks of code within a
-class using variables whose values cannot be accessed outside of the lambda.
+local to the lambda, you can use the `with` function to create private blocks
+of code within a class using variables whose values cannot be accessed outside
+of the lambda.
 
 **Example**: Using `with`
 
@@ -24,5 +24,5 @@ $check_var = $x
 - Since 4.0.0
 DOC
 ) do |args|
-  Error.is4x('with')
+  Puppet::Parser::Functions::Error.is4x('with')
 end
