@@ -70,7 +70,7 @@ Puppet::Type.type(:package).provide :portage, :parent => Puppet::Provider::Packa
     name = qatom[:pfx] + name if qatom[:pfx]
     name = name + '-' + qatom[:pv] if qatom[:pv]
     name = name + '-' + qatom[:pr] if qatom[:pr]
-    name = name + qatom[:slot] if qatom[:slot]
+    name = name + ':' + qatom[:slot] if qatom[:slot]
     cmd << '--update' if [:latest].include?(should)
     cmd += install_options if @resource[:install_options]
     cmd << name
@@ -84,7 +84,7 @@ Puppet::Type.type(:package).provide :portage, :parent => Puppet::Provider::Packa
     name = qatom[:pfx] + name if qatom[:pfx]
     name = name + '-' + qatom[:pv] if qatom[:pv]
     name = name + '-' + qatom[:pr] if qatom[:pr]
-    name = name + qatom[:slot] if qatom[:slot]
+    name = name + ':' + qatom[:slot] if qatom[:slot]
     cmd += uninstall_options if @resource[:uninstall_options]
     cmd << name
     if [:purged].include?(should)
@@ -117,7 +117,7 @@ Puppet::Type.type(:package).provide :portage, :parent => Puppet::Provider::Packa
       if match
         result_fields.zip(match.captures) do |field, value|
           # some fields can be empty or (null) (if we are not passed a category in the package name for instance)
-          if value == '(null)'
+          if value == '(null)' || value == '<unset>'
             package_info[field] = nil
           elsif !value or value.empty?
             package_info[field] = nil
@@ -133,7 +133,7 @@ Puppet::Type.type(:package).provide :portage, :parent => Puppet::Provider::Packa
   end
 
   def qatom_output_format
-    '"[%{CATEGORY}] [%{PN}] [%{PV}] [%[PR]] [%[SLOT]] [%[pfx]] [%[sfx]]"'
+    '"[%[CATEGORY]] [%[PN]] [%[PV]] [%[PR]] [%[SLOT]] [%[pfx]] [%[sfx]]"'
   end
 
   def qatom_result_format
@@ -169,7 +169,7 @@ Puppet::Type.type(:package).provide :portage, :parent => Puppet::Provider::Packa
         package_sets << package_set.to_s.strip
       end
 
-      if @resource[:name].match(/^@/)
+      if @resource[:name] =~ /^@/
         if package_sets.include?(@resource[:name][1..-1].to_s)
           return({:name => "#{@resource[:name]}", :ensure => '9999', :version_available => nil, :installed_versions => nil, :installable_versions => "9999,"})
         end
@@ -247,7 +247,7 @@ Puppet::Type.type(:package).provide :portage, :parent => Puppet::Provider::Packa
     # [2.7.12: 2.7
     #  3.4.5:  3.4
     #  3.5.2:  3.5]
-    version_for_slot = versions_and_slots.find { |version_and_slot| version_and_slot.last == slot[1..-1] }
+    version_for_slot = versions_and_slots.find { |version_and_slot| version_and_slot.last == slot }
     # [3.5.2:  3.5]
     version_for_slot.first if version_for_slot
     # 3.5.2
