@@ -1,5 +1,3 @@
-require 'puppet/indirector/data_binding/hiera'
-
 require 'tmpdir'
 require 'fileutils'
 
@@ -145,9 +143,14 @@ module Puppet::Test
       Puppet::Application.clear!
       Puppet::Util::Profiler.clear
 
-      Puppet.clear_deprecation_warnings
+      Puppet::SSL::Host.reset
+      Puppet::SSL::Host.ca_location = :none
 
-      Puppet::DataBinding::Hiera.instance_variable_set("@hiera", nil)
+      Puppet::Node::Facts.indirection.terminus_class = :memory
+      facts = Puppet::Node::Facts.new(Puppet[:node_name_value])
+      Puppet::Node::Facts.indirection.save(facts)
+
+      Puppet.clear_deprecation_warnings
     end
 
     # Call this method once per test, after execution of each individual test.
@@ -225,9 +228,6 @@ module Puppet::Test
       # Initialize "app defaults" settings to a good set of test values
       Puppet.settings.initialize_app_defaults(app_defaults_for_tests)
 
-      # Avoid opening ports to the outside world
-      Puppet.settings[:bindaddress] = "127.0.0.1"
-
       # We don't want to depend upon the reported domain name of the
       # machine running the tests, nor upon the DNS setup of that
       # domain.
