@@ -40,8 +40,8 @@ module Puppet::Util::InstanceLoader
     instance_loader(type).loadall
 
     # Use this method so they all get loaded
-    loaded_instances(type).sort { |a,b| a.to_s <=> b.to_s }.each do |name|
-      mod = self.loaded_instance(name)
+    loaded_instances(type).sort_by(&:to_s).each do |name|
+      mod = self.loaded_instance(type, name)
       docs << "#{name}\n#{"-" * name.to_s.length}\n"
 
       docs << Puppet::Util::Docs.scrub(mod.doc) << "\n\n"
